SystemOrganization addCategory: #OMeta2!
SystemOrganization addCategory: #'OMeta2-Info'!

!CompiledMethod methodsFor: '*OMeta2-Preload-override' stamp: 'bf 3/19/2012 20:11'!
methodNode
	"Return the parse tree that represents self. If parsing fails, decompile the method."
	| aClass source |
	aClass := self methodClass.
	source := self
				getSourceFor: (self selector ifNil: [self defaultSelector])
				in: aClass.
	^[(aClass parserClass new
		encoderClass: (self isBlueBookCompiled
						ifTrue: [EncoderForV3]
						ifFalse: [EncoderForV3PlusClosures]);
		parse: source class: aClass)
			sourceText: source;
			yourself]
		on: SyntaxErrorNotification
		do: [:ex | ex return: self decompileWithTemps].! !

Exception subclass: #OM2Fail
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OM2Fail class methodsFor: 'class initialization' stamp: 'hmm 4/16/2010 17:07'!
initialize
	(Smalltalk at: #OMeta2Fail ifAbsent: []) class == self
		ifFalse: [Smalltalk at: #OMeta2Fail put: self new]! !

!OM2Fail methodsFor: 'priv handling' stamp: 'warf 2/26/2009 11:03'!
defaultAction

	self error: 'match failed'! !

MethodReference subclass: #OM2DecompilingMethodReference
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2-Info'!

!OM2DecompilingMethodReference commentStamp: 'hmm 5/24/2010 13:15' prior: 0!
This class is a helper to OM2PreloadPackagingInfo which is used to save OMeta2 productions in their decompiled form.!

!OM2DecompilingMethodReference methodsFor: 'queries' stamp: 'hmm 4/16/2010 16:38'!
source
	^self compiledMethod decompileString! !

Compiler subclass: #OMeta2Compiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2Compiler methodsFor: 'as yet unclassified' stamp: 'warf 2/18/2009 16:30'!
compile: origCode in: cls classified: aCategory notifying: notify ifFail: failBlock

	^ self compile: origCode in: cls notifying: notify ifFail: failBlock! !

!OMeta2Compiler methodsFor: 'as yet unclassified' stamp: 'mg 5/12/2014 09:49'!
compile: origCode in: cls notifying: notify ifFail: failBlock

	| origCodeStream parseTree structuredCode translatedCode |
	origCodeStream := origCode asString readStream.
	self from: origCodeStream class: cls notifying: notify.
	[
		parseTree := OMeta2RuleParser matchStream: origCodeStream with: #rule withArgs: #() withPlaybackDebugging: false.
		parseTree := OMeta2Optimizer match: parseTree with: #optimizeRule.
		structuredCode := OMeta2RuleTranslator match: parseTree with: #translate withArgs: {cls}.
		translatedCode := OMeta2Flattener match: structuredCode with: #flatten
	] on: OM2Fail do: [
		self notify: '<-- parse error around here -->' at: origCodeStream position.
		^ failBlock value
	].
	^ Compiler new
		compile: translatedCode readStream
		in: cls
		notifying: notify
		ifFail: failBlock! !

!OMeta2Compiler methodsFor: 'as yet unclassified' stamp: 'hmm 8/25/2012 14:34'!
parse: aStream class: aClass noPattern: noPattern context: ctxt notifying: req ifFail: aBlock
	| node |
	^ [
		| parseTree structuredCode code |
		parseTree := OMeta2RuleParser matchAll: aStream contents with: #rule.
		structuredCode := OMeta2RuleTranslator match: parseTree with: #translate withArgs: {aClass}.
		code := OMeta2Flattener match: structuredCode with: #flatten.
		node := Parser new parse: code readStream class: aClass noPattern: noPattern context: ctxt notifying: req ifFail: aBlock.
		OMeta2MethodNode adoptInstance: node.
		node
	] on: OM2Fail do: [aBlock value]! !

!OMeta2Compiler methodsFor: 'as yet unclassified' stamp: 'hmm 5/24/2010 10:51'!
parser
	^ self parserClass new! !

!OMeta2Compiler methodsFor: 'as yet unclassified' stamp: 'warf 2/27/2009 13:10'!
parserClass

	^ self class! !

PackageInfo subclass: #OM2PostloadPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2-Info'!

!OM2PostloadPackageInfo commentStamp: 'hmm 4/16/2010 17:09' prior: 0!
This class stores all OMeta2 production rules in their original format.!

!OM2PostloadPackageInfo class methodsFor: 'class initialization' stamp: 'hmm 4/16/2010 17:13'!
initialize
	self new register! !

!OM2PostloadPackageInfo methodsFor: 'listing' stamp: 'hmm 4/16/2010 16:52'!
classes
	^super classes select: [:each | each inheritsFrom: OMeta2Base]! !

!OM2PostloadPackageInfo methodsFor: 'testing' stamp: 'hmm 4/16/2010 17:27'!
coreMethodsForClass: aClass
	MCMethodDefinition shutDown.	"flush cache so pre- and postload don't interfere"
	^(super coreMethodsForClass: aClass) select: [:each | OMeta2RuleParser isOMeta2Rule: each source]! !

!OM2PostloadPackageInfo methodsFor: 'testing' stamp: 'hmm 4/16/2010 16:50'!
extensionMethodsForClass: aClass
	^#()! !

!OM2PostloadPackageInfo methodsFor: 'naming' stamp: 'hmm 4/16/2010 16:47'!
packageName
	^super packageName, '-Postload'! !

!OM2PostloadPackageInfo methodsFor: 'naming' stamp: 'hmm 4/16/2010 16:54'!
systemCategoryPrefix
	^super packageName! !

PackageInfo subclass: #OM2PreloadPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2-Info'!

!OM2PreloadPackageInfo commentStamp: 'hmm 4/16/2010 17:08' prior: 0!
This class makes it possible to store OMeta2 using decompiled production rules, so that it can be loaded without being present first.
It uses OM2DecompilingMethodReference to decompile OMeta2 rules on the fly while creating a source file.!

!OM2PreloadPackageInfo class methodsFor: 'class initialization' stamp: 'hmm 4/16/2010 17:13'!
initialize
	self new register! !

!OM2PreloadPackageInfo methodsFor: 'naming' stamp: 'hmm 4/16/2010 16:10'!
packageName
	^super packageName, '-Preload'! !

!OM2PreloadPackageInfo methodsFor: 'listing' stamp: 'hmm 4/16/2010 17:27'!
referenceForMethod: aSymbol ofClass: aClass
	MCMethodDefinition shutDown.	"flush cache so pre- and postload don't interfere"
	^((OMeta2RuleParser isOMeta2Rule: (aClass sourceCodeAt: aSymbol))
		ifTrue: [OM2DecompilingMethodReference]
		ifFalse: [MethodReference]) new setStandardClass: aClass methodSymbol: aSymbol! !

!OM2PreloadPackageInfo methodsFor: 'naming' stamp: 'hmm 4/16/2010 16:12'!
systemCategoryPrefix
	^super packageName! !

Object subclass: #OM2Failer
	instanceVariableNames: 'used'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OM2Failer methodsFor: 'initialize-release' stamp: 'warf 2/16/2009 14:23'!
initialize

	used := false! !

!OM2Failer methodsFor: 'testing' stamp: 'warf 2/27/2009 13:44'!
used

	^ used! !

!OM2Failer methodsFor: 'evaluating' stamp: 'warf 2/27/2009 13:42'!
value

	used := true.
	OMeta2Fail signal! !

Object subclass: #OM2Stream
	instanceVariableNames: 'head tail memo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

OM2Stream subclass: #OM2EndOfStream
	instanceVariableNames: 'stream pos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OM2EndOfStream methodsFor: 'accessing' stamp: 'warf 2/25/2009 11:29'!
head

	OMeta2Fail signal! !

!OM2EndOfStream methodsFor: 'initialize-release' stamp: 'warf 2/20/2009 14:50'!
initStream: s pos: p

	stream := s.
	pos := p! !

!OM2EndOfStream methodsFor: 'accessing' stamp: 'warf 2/19/2009 23:14'!
inputSpecies

	^ stream originalContents species! !

!OM2EndOfStream methodsFor: 'accessing' stamp: 'warf 2/20/2009 14:49'!
pos

	^ pos! !

!OM2EndOfStream methodsFor: 'accessing' stamp: 'warf 2/25/2009 11:29'!
tail

	OMeta2Fail signal! !

OM2Stream subclass: #OM2LazyStream
	instanceVariableNames: 'stream pos'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OM2LazyStream class methodsFor: 'as yet unclassified' stamp: 'warf 2/20/2009 14:44'!
for: aReadStream withPos: pos

	^ aReadStream atEnd
		ifTrue: [OM2EndOfStream new initStream: aReadStream pos: pos]
		ifFalse: [self new initHead: aReadStream next stream: aReadStream pos: pos]! !

!OM2LazyStream methodsFor: 'initialize-release' stamp: 'warf 2/20/2009 14:50'!
initHead: h stream: s pos: p

	head := h.
	stream := s.
	pos := p! !

!OM2LazyStream methodsFor: 'accessing' stamp: 'warf 2/19/2009 23:15'!
inputSpecies

	^ stream originalContents species! !

!OM2LazyStream methodsFor: 'accessing' stamp: 'warf 2/20/2009 14:43'!
pos

	^ pos! !

!OM2LazyStream methodsFor: 'accessing' stamp: 'warf 2/20/2009 14:42'!
tail

	tail ifNil: [tail := OM2LazyStream for: stream withPos: pos + 1].
	^ tail! !

!OM2Stream methodsFor: 'accessing' stamp: 'warf 2/26/2009 11:04'!
basicTail

	^ tail! !

!OM2Stream methodsFor: 'forgetting' stamp: 'warf 2/26/2009 11:38'!
forgetEverything

	memo := IdentityDictionary new! !

!OM2Stream methodsFor: 'accessing' stamp: 'warf 2/16/2009 11:35'!
head

	^ head! !

!OM2Stream methodsFor: 'initialize-release' stamp: 'warf 2/16/2009 13:38'!
initHead: h tail: t

	head := h.
	tail := t! !

!OM2Stream methodsFor: 'initialize-release' stamp: 'warf 2/16/2009 11:34'!
initialize

	memo := IdentityDictionary new! !

!OM2Stream methodsFor: 'accessing' stamp: 'warf 2/19/2009 23:13'!
inputSpecies

	^ Array! !

!OM2Stream methodsFor: 'accessing' stamp: 'warf 2/16/2009 14:13'!
memo

	^ memo! !

!OM2Stream methodsFor: 'accessing' stamp: 'warf 2/20/2009 14:51'!
pos

	^ -1! !

!OM2Stream methodsFor: 'printing' stamp: 'warf 2/26/2009 11:05'!
printOn: aStream

	| inputIsString curr |
	inputIsString := (self inputSpecies inheritsFrom: String) and: [(self inputSpecies inheritsFrom: Symbol) not].
	curr := self.
	aStream
		nextPutAll: 'an ';
		nextPutAll: self class name;
		nextPut: $(.
	[curr notNil] whileTrue: [
		(curr isKindOf: OM2EndOfStream) ifTrue: [
			aStream nextPut: $).
			^ self
		].
		inputIsString
			ifTrue: [aStream nextPut: curr head]
			ifFalse: [
				curr head printOn: aStream.
				aStream space
			].
		curr := curr basicTail.
	].
	aStream nextPutAll: '...)'! !

!OM2Stream methodsFor: 'accessing' stamp: 'warf 2/16/2009 11:35'!
tail

	^ tail! !

!OM2Stream methodsFor: 'forgetting' stamp: 'warf 2/26/2009 11:40'!
transitiveForgetEverything

	| curr |
	curr := self.
	[curr notNil] whileTrue: [
		curr forgetEverything.
		curr := curr basicTail
	]! !

OM2Stream subclass: #OM2StreamDebugger
	instanceVariableNames: 'om2stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OM2StreamDebugger class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 15:28'!
for: anOM2Stream

	^ self new initOm2stream: anOM2Stream! !

!OM2StreamDebugger methodsFor: 'forgetting' stamp: 'warf 2/27/2009 13:28'!
forgetEverything

	om2stream forgetEverything! !

!OM2StreamDebugger methodsFor: 'accessing' stamp: 'warf 2/26/2009 13:40'!
head

	^ om2stream head! !

!OM2StreamDebugger methodsFor: 'initialize-release' stamp: 'warf 2/26/2009 15:26'!
initOm2stream: anOM2Stream

	om2stream := anOM2Stream! !

!OM2StreamDebugger methodsFor: 'accessing' stamp: 'warf 2/27/2009 13:27'!
memo

	^ om2stream memo! !

!OM2StreamDebugger methodsFor: 'printing' stamp: 'warf 2/26/2009 15:57'!
printOn: aStream

	aStream nextPutAll: 'an OM2StreamDebugger('.
	om2stream printOn: aStream.
	aStream nextPut: $)! !

!OM2StreamDebugger methodsFor: 'accessing' stamp: 'warf 2/26/2009 13:40'!
tail

	^ om2stream tail! !

!OM2StreamDebugger methodsFor: 'forgetting' stamp: 'warf 2/27/2009 13:29'!
transitiveForgetEverything

	om2stream transitiveForgetEverything! !

OM2Stream subclass: #OM2StreamProxy
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OM2StreamProxy class methodsFor: 'as yet unclassified' stamp: 'warf 2/19/2009 23:44'!
for: anOM2Stream

	^ self new initTarget: anOM2Stream! !

!OM2StreamProxy methodsFor: 'accessing' stamp: 'warf 2/26/2009 11:05'!
basicTail

	^ target basicTail! !

!OM2StreamProxy methodsFor: 'accessing' stamp: 'warf 2/16/2009 12:08'!
head

	head ifNil: [head := target head].
	^ head
! !

!OM2StreamProxy methodsFor: 'initialize-release' stamp: 'warf 2/19/2009 23:45'!
initTarget: anOM2Stream

	target := anOM2Stream! !

!OM2StreamProxy methodsFor: 'accessing' stamp: 'warf 2/19/2009 23:13'!
inputSpecies

	^ target inputSpecies! !

!OM2StreamProxy methodsFor: 'accessing' stamp: 'warf 2/20/2009 14:51'!
pos

	^ target pos! !

!OM2StreamProxy methodsFor: 'accessing' stamp: 'warf 2/16/2009 12:07'!
tail

	tail ifNil: [tail := OM2StreamProxy for: target tail].
	^ tail! !

!OM2StreamProxy methodsFor: 'accessing' stamp: 'warf 2/16/2009 14:10'!
target

	^ target! !

Object subclass: #OMeta2Base
	instanceVariableNames: 'input om2streams haltingPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

OMeta2Base subclass: #OMeta2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2 commentStamp: '<historical>' prior: 0!
TODOS:

* implement OMeta -> Squeak translator
* implement Squeak parser
* implement OMeta/Squeak "compiler", make it OMeta2's compilerClass

* rewrite #char, #digit, #empty, #end, #exactly, #firstAndRest, #fromTo, #letter, #letterOrDigit, #listOf, #lower, #notLast, #number, #range, #space, #spaces, #string, #symbol, #token, and #upper in OMeta syntax
* consider implementing position-related functionality (see http://www.tinlizzie.org/ometa-js/ometa-base.js)
* consider the optimization suggestions in the comments of OMeta2Lib's methods!

OMeta2 subclass: #O2SqueakRecognizer
	instanceVariableNames: ''
	classVariableNames: 'TypeTable'
	poolDictionaries: ''
	category: 'OMeta2'!

!O2SqueakRecognizer class methodsFor: 'as yet unclassified' stamp: 'warf 2/20/2009 02:04'!
initialize

	TypeTable := #(#xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xDelimiter #xDelimiter #xBinary #xDelimiter #xDelimiter #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #doIt #xBinary #xDelimiter #xBinary #xDoubleQuote #xLitQuote #xDollar #xBinary #xBinary #xSingleQuote #leftParenthesis #rightParenthesis #xBinary #xBinary #xBinary #xBinary #period #xBinary #xDigit #xDigit #xDigit #xDigit #xDigit #xDigit #xDigit #xDigit #xDigit #xDigit #xColon #semicolon #xBinary #xBinary #xBinary #xBinary #xBinary #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #leftBracket #xBinary #rightBracket #upArrow #leftArrow #xBinary #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #leftBrace #verticalBar #rightBrace #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xLetter #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xBinary #xLetter #xBinary #xBinary #xBinary #xBinary #xLetter #xBinary #xBinary #xBinary #xBinary #xBinary #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xBinary #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xBinary #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xLetter #xBinary)! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:43'!
arrayConstr
	^ self ometaOr: {[true
			ifTrue: [self apply: #token withArgs: {'{'}.
				self apply: #expr.
				self
					many: [true
							ifTrue: [self apply: #token withArgs: {'.'}.
								self apply: #expr]].
				self ometaOr: {[self apply: #token withArgs: {'.'}]. [self apply: #empty]}.
				self apply: #token withArgs: {'}'}]]. [true
			ifTrue: [self apply: #token withArgs: {'{'}.
				self apply: #token withArgs: {'}'}]]}! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:57'!
arrayLit
	^ true
		ifTrue: [self apply: #token withArgs: {'#'}.
			self apply: #token withArgs: {'('}.
			self
				many: [self ometaOr: {[self apply: #literal]. [self apply: #arrayLit]. [true
							ifTrue: [self apply: #spaces.
								self apply: #tsArraySymbol]]}].
			self apply: #token withArgs: {')'}]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:57'!
binary
	^ true
		ifTrue: [self apply: #spaces.
			self apply: #tsBinary]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:58'!
binaryExpr
	^ self ometaOr: {[true
			ifTrue: [self apply: #binaryExpr.
				self apply: #binaryMsg]]. [self apply: #unaryExpr]}! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:58'!
binaryMsg
	^ true
		ifTrue: [self apply: #binary.
			self apply: #unaryExpr]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:08'!
block
	^ true
		ifTrue: [self apply: #token withArgs: {'['}.
			self ometaOr: {[true
					ifTrue: [self
							many1: [true
									ifTrue: [self apply: #token withArgs: {':'}.
										self apply: #identifier]].
						self apply: #token withArgs: {'|'}]]. [self apply: #empty]}.
			self ometaOr: {[true
					ifTrue: [self apply: #token withArgs: {'|'}.
						self
							many: [self apply: #identifier].
						self apply: #token withArgs: {'|'}]]. [self apply: #empty]}.
			self ometaOr: {[true
					ifTrue: [self apply: #expr.
						self
							many: [true
									ifTrue: [self apply: #token withArgs: {'.'}.
										self apply: #expr]].
						self ometaOr: {[true
								ifTrue: [self apply: #token withArgs: {'.'}.
									self apply: #token withArgs: {'^'}.
									self apply: #expr]]. [self apply: #empty]}]]. [true
					ifTrue: [self apply: #token withArgs: {'^'}.
						self apply: #expr]]. [self apply: #empty]}.
			self ometaOr: {[self apply: #token withArgs: {'.'}]. [self apply: #empty]}.
			self apply: #token withArgs: {']'}]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:58'!
cascade
	^ self ometaOr: {[self apply: #identifier]. [self apply: #binaryMsg]. [self apply: #keywordMsg]}! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 00:59'!
expr
	^ self ometaOr: {[true
			ifTrue: [self apply: #identifier.
				self ometaOr: {[self apply: #token withArgs: {':='}]. [self apply: #token withArgs: {'_'}]}.
				self apply: #expr]]. [self apply: #msgExpr]}! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:18'!
identifier
	^ true
		ifTrue: [self apply: #spaces.
			self apply: #tsIdentifier.
			self
				not: [self apply: #exactly withArgs: {$:}]]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:19'!
keyword
	^ true
		ifTrue: [self apply: #spaces.
			self apply: #tsKeyword]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:19'!
keywordExpr
	^ true
		ifTrue: [self apply: #binaryExpr.
			self apply: #keywordMsg]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:19'!
keywordMsg
	^ self ometaOr: {[true
			ifTrue: [self apply: #keywordMsg.
				self apply: #keywordMsgPart]]. [self apply: #keywordMsgPart]}! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:20'!
keywordMsgPart
	^ true
		ifTrue: [self apply: #keyword.
			self apply: #binaryExpr]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 4/16/2010 13:38'!
literal
	^ true
		ifTrue: [self apply: #spaces.
			self ometaOr: {[self apply: #tsNumber]. [self apply: #tsCharacter]. [self apply: #tsString]. [self apply: #tsSymbol]}]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:21'!
msgExpr
	^ true
		ifTrue: [self ometaOr: {[self apply: #keywordExpr]. [self apply: #binaryExpr]}.
			self
				many: [true
						ifTrue: [self apply: #token withArgs: {';'}.
							self apply: #cascade]]]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:30'!
squeakExpr
	^ self
		consumedBy: [self apply: #expr]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:23'!
symbol
	^ true
		ifTrue: [self apply: #token withArgs: {'#'}.
			self apply: #spaces.
			self ometaOr: {[self apply: #tsString]. [true
					ifTrue: [self apply: #tsKeyword.
						self ometaOr: {[self apply: #tsIdentifier]. [self apply: #empty]}]]}]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:23'!
tcBinaryChar
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: (TypeTable at: t1 asciiValue)
					== #xBinary]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:25'!
tsArraySymbol
	^ self ometaOr: {[true
			ifTrue: [self
					many1: [self apply: #tsKeyword].
				self ometaOr: {[self apply: #tsIdentifier]. [self apply: #empty]}]]. [self apply: #tsIdentifier]}! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:25'!
tsBinary
	^ true
		ifTrue: [self ometaOr: {[self apply: #exactly withArgs: {$|}]. [self apply: #tcBinaryChar]}.
			self
				many: [self apply: #tcBinaryChar]]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:25'!
tsCharacter
	^ true
		ifTrue: [self apply: #exactly withArgs: {$$}.
			self apply: #char]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:26'!
tsIdentifier
	^ true
		ifTrue: [self apply: #letter.
			self
				many: [self ometaOr: {[self apply: #letter]. [self apply: #digit]}]]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:31'!
tsKeyword
	^ true
		ifTrue: [self apply: #tsIdentifier.
			self apply: #exactly withArgs: {$:}]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:26'!
tsNatural
	^ self
		many1: [self apply: #digit]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:27'!
tsNumber
	^ true
		ifTrue: [self ometaOr: {[self apply: #exactly withArgs: {$+}]. [self apply: #exactly withArgs: {$-}]. [self apply: #empty]}.
			self apply: #tsNatural]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:28'!
tsString
	^ true
		ifTrue: [self apply: #exactly withArgs: {$'}.
			self
				many: [self ometaOr: {[true
							ifTrue: [self apply: #exactly withArgs: {$'}.
								self apply: #exactly withArgs: {$'}]]. [true
							ifTrue: [self
									not: [self apply: #exactly withArgs: {$'}].
								self apply: #char]]}].
			self apply: #exactly withArgs: {$'}]! !

!O2SqueakRecognizer methodsFor: 'rules-lexing' stamp: 'warf 2/20/2009 01:28'!
tsSymbol
	^ true
		ifTrue: [self apply: #exactly withArgs: {$#}.
			self apply: #spaces.
			self ometaOr: {[self apply: #tsString]. [self apply: #tsArraySymbol]}]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:28'!
unaryExpr
	^ true
		ifTrue: [self apply: #unit.
			self
				many: [self apply: #identifier]]! !

!O2SqueakRecognizer methodsFor: 'rules-parsing' stamp: 'warf 2/20/2009 01:29'!
unit
	^ self ometaOr: {[self apply: #literal]. [self apply: #identifier]. [self apply: #arrayLit]. [self apply: #arrayConstr]. [self apply: #block]. [true
			ifTrue: [self apply: #token withArgs: {'('}.
				self apply: #expr.
				self apply: #token withArgs: {')'}]]}! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:28'!
char
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self pred: t1 isCharacter.
			t1]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:28'!
digit
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: t1 isDigit.
			t1]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:29'!
end
	^ self
		not: [self apply: #anything]! !

!OMeta2 methodsFor: 'rules-meta' stamp: 'warf 4/3/2010 00:29'!
exactly
	| t1 t2 |
	^ true
		ifTrue: [t2 := self apply: #anything.
			t1 := self apply: #anything.
			self pred: t2 = t1.
			t2]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:30'!
fromTo
	| t1 t2 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			t2 := self apply: #anything.
			self apply: #seq withArgs: {t1}.
			self
				many: [true
						ifTrue: [self
								not: [self apply: #seq withArgs: {t2}].
							self apply: #char]].
			self apply: #seq withArgs: {t2}]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:31'!
letter
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: t1 isLetter.
			t1]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:31'!
letterOrDigit
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: t1 isAlphaNumeric.
			t1]! !

!OMeta2 methodsFor: 'rules-meta' stamp: 'warf 4/13/2010 12:12'!
listOf
	| t1 t2 t3 t5 |
	^ true
		ifTrue: [t2 := self apply: #anything.
			t1 := self apply: #anything.
			self ometaOr: {[true
					ifTrue: [t3 := self apply: #apply withArgs: {t2}.
						t5 := self
									many: [true
											ifTrue: [self apply: #token withArgs: {t1}.
												self apply: #apply withArgs: {t2}]].
						t5 addFirst: t3;
							 yourself]]. [true
					ifTrue: [self apply: #empty.
						#()]]}]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:31'!
lower
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: t1 isLowercase.
			t1]! !

!OMeta2 methodsFor: 'rules-meta' stamp: 'warf 4/3/2010 00:34'!
notLast
	| t1 t2 |
	^ true
		ifTrue: [t2 := self apply: #anything.
			t1 := self apply: #apply withArgs: {t2}.
			self
				lookahead: [self apply: #apply withArgs: {t2}].
			t1]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:34'!
number
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self pred: t1 isNumber.
			t1]! !

!OMeta2 methodsFor: 'rules-meta' stamp: 'yo 5/5/2011 10:26'!
range
	| t1 t2 t3 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			t2 := self apply: #anything.
			t3 := self apply: #anything.
			self
				pred: (t1 <= t3
						and: [t3 <= t2]).
			t3]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:35'!
space
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: t1 asciiValue <= 32.
			t1]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:35'!
spaces
	^ self
		many: [self apply: #space]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:36'!
string
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self pred: t1 isString.
			t1]! !

!OMeta2 methodsFor: 'rules' stamp: 'warf 4/3/2010 00:36'!
symbol
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self pred: t1 isSymbol.
			t1]! !

!OMeta2 methodsFor: 'rules-meta' stamp: 'warf 4/3/2010 00:36'!
token
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self apply: #spaces.
			self apply: #seq withArgs: {t1}]! !

!OMeta2 methodsFor: 'rules' stamp: 'hmm 5/24/2010 10:59'!
upper
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #char.
			self pred: t1 isUppercase.
			t1]! !

OMeta2 subclass: #OMeta2Examples
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2Examples commentStamp: '<historical>' prior: 0!
Here's how to run these examples:

	OMeta2Examples match: 5 with: #fact.
	OMeta2Examples matchAll: '1234' with: #number.
	OMeta2Examples matchAll: 'abc123' with: #identifier.
	OMeta2Examples matchAll: #($a $b $c 1 2 3 #(4 5)) with: #structure.
	OMeta2Examples matchAll: 'howdy' with: #greeting.!

!OMeta2Examples methodsFor: 'squeak methods' stamp: 'warf 2/20/2009 02:35'!
aSqueakMethod

	self inform: 'hello world'! !

!OMeta2Examples methodsFor: 'squeak methods' stamp: 'warf 2/20/2009 02:36'!
anotherSqueakMethod

	self inform: 'good-bye world'! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/27/2009 15:58'!
digit
	| t1 |
	^ true
		ifTrue: [t1 := self
						super: OMeta2
						apply: #digit
						withArgs: {}.
			t1 digitValue]! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 5/20/2009 12:58'!
fact
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [self apply: #exactly withArgs: {0}.
				1]]. [true
			ifTrue: [t3 := self apply: #anything.
				t1 := self apply: #fact withArgs: {t3 - 1}.
				t3 * t1]]}! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/26/2009 12:04'!
formTest
	^ self ometaOr: {[self
			form: [true
					ifTrue: [self inform: input hash printString.
						self apply: #exactly withArgs: {#foo}.
						self apply: #exactly withArgs: {#bar}]]]. [self
			form: [true
					ifTrue: [self inform: input hash printString.
						self apply: #exactly withArgs: {#bar}]]]}! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/20/2009 14:36'!
greeting
	^ self
		consumedBy: [true
				ifTrue: [self apply: #seq withArgs: {'howdy'}.
					self
						opt: [self apply: #seq withArgs: {'-ho'}]]]! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/27/2009 15:45'!
identifier
	^ true
		ifTrue: [self apply: #spaces.
			self
				consumedBy: [true
						ifTrue: [self apply: #letter.
							self
								many: [self ometaOr: {[self apply: #letter]. [self apply: #digit]}]]]]! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/27/2009 15:48'!
identifierIdx
	^ true
		ifTrue: [self apply: #spaces.
			self
				indexConsumedBy: [true
						ifTrue: [self apply: #letter.
							self
								many: [self ometaOr: {[self apply: #letter]. [self apply: #digit]}]]]]! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/20/2009 14:54'!
identifiers
	^ self
		many: [true
				ifTrue: [self inform: self pos printString.
					self apply: #identifier]]! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/27/2009 15:46'!
number
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [t3 := self apply: #number.
				t1 := self apply: #digit.
				t3 * 10 + t1]]. [self apply: #digit]}! !

!OMeta2Examples methodsFor: 'rules' stamp: 'warf 2/27/2009 15:46'!
structure
	^ true
		ifTrue: [self apply: #exactly withArgs: {$a}.
			self apply: #exactly withArgs: {$b}.
			self apply: #exactly withArgs: {$c}.
			self apply: #exactly withArgs: {1}.
			self apply: #exactly withArgs: {2}.
			self apply: #exactly withArgs: {3}.
			self
				form: [self
						many: [self apply: #anything]]]! !

OMeta2 subclass: #OMeta2Flattener
	instanceVariableNames: 'ws'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2Flattener methodsFor: 'rules' stamp: 'warf 4/3/2010 00:40'!
flatten
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self apply: #iFlatten withArgs: {t1}.
			ws contents]! !

!OMeta2Flattener methodsFor: 'rules' stamp: 'warf 4/3/2010 00:41'!
iFlatten
	| t1 |
	^ self ometaOr: {[true
			ifTrue: [t1 := self apply: #string.
				ws nextPutAll: t1]]. [self
			form: [self
					many: [self apply: #iFlatten]]]}! !

!OMeta2Flattener methodsFor: 'initialize-release' stamp: 'warf 2/18/2009 15:05'!
initialize

	super initialize.
	ws := (String new: 64) writeStream! !

OMeta2 subclass: #OMeta2NullOpt
	instanceVariableNames: 'didSomething'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

OMeta2NullOpt subclass: #OMeta2AndOrOpt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2AndOrOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:46'!
and
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [t1 := self apply: #trans.
				self apply: #end.
				self apply: #setHelped.
				t1]]. [true
			ifTrue: [t3 := self apply: #transInside withArgs: {#And}.
				t3 addFirst: #And;
					 yourself]]}! !

!OMeta2AndOrOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:46'!
or
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [t1 := self apply: #trans.
				self apply: #end.
				self apply: #setHelped.
				t1]]. [true
			ifTrue: [t3 := self apply: #transInside withArgs: {#Or}.
				t3 addFirst: #Or;
					 yourself]]}! !

!OMeta2AndOrOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:36'!
transInside
	| t1 t2 t4 t5 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self ometaOr: {[true
					ifTrue: [self
							form: [true
									ifTrue: [self apply: #exactly withArgs: {t1}.
										t4 := self apply: #transInside withArgs: {t1}]].
						t5 := self apply: #transInside withArgs: {t1}.
						self apply: #setHelped.
						t4 , t5]]. [true
					ifTrue: [t2 := self apply: #trans.
						t4 := self apply: #transInside withArgs: {t1}.
						t4 addFirst: t2;
							 yourself]]. [true
					ifTrue: [self apply: #empty.
						OrderedCollection new]]}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
and
	| t1 |
	^ true
		ifTrue: [t1 := self
						many: [self apply: #trans].
			t1 addFirst: #And;
				 yourself]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
consby
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#ConsBy. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
form
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#Form. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:01'!
helped
	^ self pred: didSomething! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
idxconsby
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#IdxConsBy. t1}]! !

!OMeta2NullOpt methodsFor: 'initialize-release' stamp: 'warf 5/20/2009 12:30'!
initialize

	super initialize.
	didSomething := false! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
lookahead
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#Lookahead. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
many
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#Many. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:44'!
many1
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#Many1. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:45'!
not
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#Not. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:45'!
opt
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{#Opt. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:02'!
optimize
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			self apply: #helped.
			t1]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:45'!
or
	| t1 |
	^ true
		ifTrue: [t1 := self
						many: [self apply: #trans].
			t1 addFirst: #Or;
				 yourself]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:45'!
rule
	| t1 t2 t3 |
	^ true
		ifTrue: [t2 := self apply: #anything.
			t3 := self apply: #anything.
			t1 := self apply: #trans.
			{#Rule. t2. t3. t1}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:45'!
set
	| t1 t2 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			t2 := self apply: #trans.
			{#Set. t1. t2}]! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:00'!
setHelped
	^ didSomething := true! !

!OMeta2NullOpt methodsFor: 'rules' stamp: 'warf 5/20/2009 12:42'!
trans
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [self
					form: [true
							ifTrue: [t3 := self apply: #anything.
								t3 := t3 asLowercase asSymbol.
								self
									pred: (self class canUnderstand: t3).
								t1 := self apply: #apply withArgs: {t3}]].
				t1]]. [self apply: #anything]}! !

OMeta2 subclass: #OMeta2Optimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2Optimizer methodsFor: 'rules' stamp: 'warf 4/13/2010 12:45'!
optimizeRule
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self
				many: [t1 := self apply: #foreign withArgs: {OMeta2AndOrOpt. #optimize. t1}].
			t1]! !

OMeta2 subclass: #OMeta2RuleParser
	instanceVariableNames: 'temps'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2RuleParser class methodsFor: 'as yet unclassified' stamp: 'warf 2/23/2009 10:07'!
isOMeta2Rule: aString

	^ [(self matchAll: aString with: #rule) first ~= #Squeak] on: OM2Fail do: [false]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:44'!
application
	| t1 t3 t4 |
	^ self ometaOr: {[true
			ifTrue: [self apply: #token withArgs: {'^'}.
				t4 := self apply: #name.
				t1 := self apply: #args.
				{#SuperApp. t4} , t1]]. [true
			ifTrue: [t3 := self apply: #name.
				self apply: #exactly withArgs: {$.}.
				t4 := self apply: #nsName.
				t1 := self apply: #args.
				{#App. #foreign. t3. ('#' , t4) asSymbol} , t1]]. [true
			ifTrue: [t4 := self apply: #name.
				t1 := self apply: #args.
				{#App. t4} , t1]]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:33'!
args
	| t1 |
	^ self ometaOr: {[true
			ifTrue: [self apply: #exactly withArgs: {$(}.
				t1 := self apply: #listOf withArgs: {#squeakExpression. '.'}.
				self apply: #token withArgs: {')'}.
				t1]]. [true
			ifTrue: [self
					not: [self apply: #exactly withArgs: {$(}].
				#()]]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:16'!
characterLiteral
	| t1 |
	^ true
		ifTrue: [self apply: #spaces.
			self apply: #exactly withArgs: {$$}.
			t1 := self apply: #char.
			{#App. #exactly. t1 storeString}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:48'!
characters
	| t1 |
	^ true
		ifTrue: [self apply: #token withArgs: {'``'}.
			t1 := self
						many: [true
								ifTrue: [self
										not: [true
												ifTrue: [self apply: #exactly withArgs: {$'}.
													self apply: #exactly withArgs: {$'}]].
									self apply: #char]].
			self apply: #exactly withArgs: {$'}.
			self apply: #exactly withArgs: {$'}.
			{#App. #seq. (String withAll: t1) storeString}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/3/2010 00:55'!
expr
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #listOf withArgs: {#expr4. '|'}.
			(OrderedCollection with: #Or) addAll: t1;
				 yourself]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:51'!
expr1
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [t3 := self ometaOr: {[self apply: #keyword withArgs: {'true'}]. [self apply: #keyword withArgs: {'false'}]. [self apply: #keyword withArgs: {'nil'}]}.
				{#App. #exactly. t3}]]. [self apply: #application]. [self apply: #semanticAction]. [self apply: #semanticPredicate]. [self apply: #characters]. [self apply: #tokenSugar]. [self apply: #stringLiteral]. [self apply: #symbolLiteral]. [self apply: #numberLiteral]. [self apply: #characterLiteral]. [true
			ifTrue: [self apply: #token withArgs: {'{'}.
				t1 := self apply: #expr.
				self apply: #token withArgs: {'}'}.
				{#Form. t1}]]. [true
			ifTrue: [self apply: #token withArgs: {'<'}.
				t1 := self apply: #expr.
				self apply: #token withArgs: {'>'}.
				{#ConsBy. t1}]]. [true
			ifTrue: [self apply: #token withArgs: {'@<'}.
				t1 := self apply: #expr.
				self apply: #token withArgs: {'>'}.
				{#IdxConsBy. t1}]]. [true
			ifTrue: [self apply: #token withArgs: {'('}.
				t1 := self apply: #expr.
				self apply: #token withArgs: {')'}.
				t1]]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:52'!
expr2
	| t1 |
	^ self ometaOr: {[true
			ifTrue: [self apply: #token withArgs: {'~'}.
				t1 := self apply: #expr2.
				{#Not. t1}]]. [true
			ifTrue: [self apply: #token withArgs: {'&'}.
				t1 := self apply: #expr2.
				{#Lookahead. t1}]]. [self apply: #expr1]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 5/18/2010 12:28'!
expr3
	| t1 t3 |
	^ self ometaOr: {[true
			ifTrue: [t3 := self apply: #expr2.
				t3 := self apply: #optIter withArgs: {t3}.
				self ometaOr: {[true
						ifTrue: [self apply: #exactly withArgs: {$:}.
							t1 := self apply: #nsName.
							temps add: t1.
							{#Set. t1. t3}]]. [true
						ifTrue: [self apply: #empty.
							t3]]}]]. [true
			ifTrue: [self apply: #token withArgs: {':'}.
				t1 := self apply: #nsName.
				temps add: t1.
				{#Set. t1. {#App. #anything}}]]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:54'!
expr4
	| t1 |
	^ true
		ifTrue: [t1 := self
						many: [self apply: #expr3].
			(OrderedCollection with: #And) addAll: t1;
				 yourself]! !

!OMeta2RuleParser methodsFor: 'initialize-release' stamp: 'warf 2/18/2009 15:04'!
initialize

	super initialize.
	temps := IdentitySet new! !

!OMeta2RuleParser methodsFor: 'rules-meta' stamp: 'warf 4/13/2010 12:54'!
keyword
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self apply: #token withArgs: {t1}.
			self
				not: [self apply: #letterOrDigit].
			t1]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:33'!
name
	^ true
		ifTrue: [self apply: #spaces.
			self apply: #nsName]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:55'!
nameFirst
	^ self apply: #letter! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:55'!
nameRest
	^ self ometaOr: {[self apply: #nameFirst]. [self apply: #digit]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:56'!
nsName
	| t1 |
	^ self ometaOr: {[true
			ifTrue: [t1 := self apply: #firstAndRest withArgs: {#nameFirst. #nameRest}.
				(String withAll: t1) asSymbol]]. [true
			ifTrue: [self apply: #exactly withArgs: {$_}.
				#anything]]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:57'!
numberLiteral
	| t1 t2 |
	^ true
		ifTrue: [self apply: #spaces.
			t2 := self ometaOr: {[true
							ifTrue: [self apply: #exactly withArgs: {$-}.
								self apply: #spaces.
								-1]]. [true
							ifTrue: [self apply: #empty.
								1]]}.
			t1 := self
						many1: [self apply: #digit].
			{#App. #exactly. (t2 * (String withAll: t1) asNumber) storeString}]! !

!OMeta2RuleParser methodsFor: 'rules-meta' stamp: 'warf 4/13/2010 13:11'!
optIter
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #anything.
			self ometaOr: {[true
					ifTrue: [self apply: #token withArgs: {'*'}.
						{#Many. t1}]]. [true
					ifTrue: [self apply: #token withArgs: {'+'}.
						{#Many1. t1}]]. [true
					ifTrue: [self apply: #token withArgs: {'?'}.
						self
							not: [self apply: #exactly withArgs: {$[}].
						{#Opt. t1}]]. [true
					ifTrue: [self apply: #empty.
						t1]]}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'hmm 4/16/2010 17:43'!
rule
	| t1 t3 t4 t5 |
	^ self ometaOr: {[true
			ifTrue: [self
					not: [true
							ifTrue: [self
									many: [self
											super: OMeta2
											apply: #space
											withArgs: {}].
								self apply: #nsName.
								self apply: #expr4.
								self apply: #token withArgs: {'='}]].
				t3 := self
							consumedBy: [self
									many: [self apply: #char]].
				{#Squeak. t3}]]. [true
			ifTrue: [t1 := self
							lookahead: [true
									ifTrue: [self
											many: [self
													super: OMeta2
													apply: #space
													withArgs: {}].
										self apply: #nsName]].
				t4 := self apply: #rulePart withArgs: {t1}.
				t5 := self
							many: [true
									ifTrue: [self apply: #token withArgs: {','}.
										self apply: #rulePart withArgs: {t1}]].
				self apply: #spaces.
				self apply: #end.
				{#Rule. t1. temps asSortedCollection. (OrderedCollection with: #Or with: t4) addAll: t5;
					 yourself}]]}! !

!OMeta2RuleParser methodsFor: 'rules-meta' stamp: 'warf 4/13/2010 13:14'!
rulePart
	| t1 t2 t3 t4 |
	^ true
		ifTrue: [t3 := self apply: #anything.
			t2 := self apply: #name.
			self pred: t2 = t3.
			t1 := self apply: #expr4.
			self ometaOr: {[true
					ifTrue: [self apply: #token withArgs: {'='}.
						t4 := self apply: #expr.
						{#And. t1. t4}]]. [true
					ifTrue: [self apply: #empty.
						t1]]}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 13:14'!
semanticAction
	| t1 |
	^ true
		ifTrue: [self
				opt: [self apply: #token withArgs: {'->'}].
			self apply: #token withArgs: {'['}.
			t1 := self apply: #squeakExpression.
			self apply: #exactly withArgs: {$]}.
			{#Act. t1}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 13:14'!
semanticPredicate
	| t1 |
	^ true
		ifTrue: [self apply: #token withArgs: {'?['}.
			t1 := self apply: #squeakExpression.
			self apply: #exactly withArgs: {$]}.
			{#Pred. t1}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 13:16'!
space
	^ self ometaOr: {[self
			super: OMeta2
			apply: #space
			withArgs: {}]. [self apply: #fromTo withArgs: {'/*'. '*/'}]. [self apply: #fromTo withArgs: {'//'. String cr}]}! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 12:44'!
squeakExpression
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #foreign withArgs: {O2SqueakRecognizer. #squeakExpr}.
			self apply: #spaces.
			t1]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 13:20'!
stringLiteral
	| t1 |
	^ true
		ifTrue: [self apply: #token withArgs: {''''}.
			t1 := self
						many: [self ometaOr: {[true
									ifTrue: [self apply: #exactly withArgs: {$'}.
										self apply: #exactly withArgs: {$'}.
										$']]. [true
									ifTrue: [self
											not: [self apply: #exactly withArgs: {$'}].
										self apply: #char]]}].
			self apply: #exactly withArgs: {$'}.
			{#App. #exactly. (String withAll: t1) storeString}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 13:17'!
symbolLiteral
	| t1 |
	^ true
		ifTrue: [self apply: #token withArgs: {'#'}.
			t1 := self apply: #nsName.
			{#App. #exactly. t1 storeString}]! !

!OMeta2RuleParser methodsFor: 'rules' stamp: 'warf 4/13/2010 13:17'!
tokenSugar
	| t1 |
	^ true
		ifTrue: [self apply: #token withArgs: {'"'}.
			t1 := self
						many: [true
								ifTrue: [self
										not: [self apply: #exactly withArgs: {$"}].
									self apply: #char]].
			self apply: #exactly withArgs: {$"}.
			{#App. #token. (String withAll: t1) storeString}]! !

OMeta2 subclass: #OMeta2RuleTranslator
	instanceVariableNames: 'grammarClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:44'!
act
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #string.
			{'('. t1. ')'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:44'!
and
	| t1 |
	^ true
		ifTrue: [t1 := self
						many: [self apply: #trans].
			{'(true ifTrue: ['. self delim: t1 with: '. '. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:52'!
app
	| t1 t2 |
	^ true
		ifTrue: [t1 := self apply: #symbol.
			self ometaOr: {[true
					ifTrue: [t2 := self
									many1: [self apply: #anything].
						t2 := self delim: t2 with: '. '.
						{'(self apply: '. t1 storeString. ' withArgs: {'. t2. '})'}]]. [{'(self apply: '. t1 storeString. ')'}]}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:51'!
consby
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self consumedBy: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'helpers' stamp: 'warf 4/3/2010 00:51'!
delim: aSequenceableCollection  with: anObject

	| first ans |
	first := true.
	ans := OrderedCollection new.
	aSequenceableCollection do: [:x |
		first ifTrue: [first := false] ifFalse: [ans add: anObject].
		ans add: x
	].
	^ ans! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:50'!
form
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self form: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:50'!
idxconsby
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self indexConsumedBy: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:50'!
lookahead
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self lookahead: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:49'!
many
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self many: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:49'!
many1
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self many1: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:49'!
not
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self not: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:49'!
opt
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #trans.
			{'(self opt: ['. t1. '])'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:49'!
or
	| t1 t2 |
	^ true
		ifTrue: [t1 := self
						many: [true
								ifTrue: [t2 := self apply: #trans.
									{'['. t2. ']'}]].
			{'(self ometaOr: {'. self delim: t1 with: '. '. '})'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:48'!
pred
	| t1 |
	^ true
		ifTrue: [t1 := self apply: #string.
			{'(self pred: ('. t1. '))'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'hmm 4/17/2010 17:18'!
rule
	| t1 t2 t3 |
	^ true
		ifTrue: [t2 := self apply: #symbol.
			t3 := self apply: #anything.
			t3 := t3
						select: [:t4 | (grammarClass instVarNames includes: t4) not].
			t1 := self apply: #trans.
			{t2. ' |'. self delim: t3 asSortedCollection with: ' '. ' | ^ '. t1}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:47'!
set
	| t1 t2 |
	^ true
		ifTrue: [t1 := self apply: #symbol.
			t2 := self apply: #trans.
			{'('. t1 asString. ' := '. t2. ')'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:47'!
squeak
	^ self apply: #string! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/13/2010 13:03'!
superapp
	| t1 t2 |
	^ true
		ifTrue: [t2 := self apply: #symbol.
			t1 := self
						many: [self apply: #anything].
			t1 := self delim: t1 with: '. '.
			{'(self super: '. grammarClass superclass name. ' apply: '. t2 storeString. ' withArgs: {'. t1. '})'}]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:45'!
trans
	| t1 t3 |
	^ true
		ifTrue: [self
				form: [true
						ifTrue: [t3 := self apply: #symbol.
							t1 := self apply: #apply withArgs: {t3 asLowercase asSymbol}]].
			t1]! !

!OMeta2RuleTranslator methodsFor: 'rules' stamp: 'warf 4/3/2010 00:45'!
translate
	^ true
		ifTrue: [grammarClass := self apply: #anything.
			self apply: #trans]! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/20/2009 14:52'!
compilerClass

        ^ OMeta2Compiler! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 12:52'!
debugMatch: anObject with: aRule

	^ self debugMatch: anObject with: aRule withArgs: #()! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 12:58'!
debugMatch: anObject with: aRule withArgs: args

	^ self debugMatchAll: {anObject} readStream with: aRule withArgs: args! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 12:35'!
debugMatchAll: aSequenceableCollection with: aRule

	^ self debugMatchAll: aSequenceableCollection with: aRule withArgs: #()! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 12:53'!
debugMatchAll: aSequenceableCollection with: aRule withArgs: args

	^ self matchStream: aSequenceableCollection readStream with: aRule withArgs: args withPlaybackDebugging: true! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 11:53'!
match: anObject with: aRule

	^ self match: anObject with: aRule withArgs: #()! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 12:55'!
match: anObject with: aRule withArgs: args

	^ self matchAll: {anObject} readStream with: aRule withArgs: args! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/18/2009 15:39'!
matchAll: aSequenceableCollection with: aRule

	^ self matchAll: aSequenceableCollection with: aRule withArgs: #()! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/26/2009 12:57'!
matchAll: aSequenceableCollection with: aRule withArgs: args

	^ self matchStream: aSequenceableCollection readStream with: aRule withArgs: args withPlaybackDebugging: false! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'warf 2/27/2009 14:14'!
matchStream: aReadStream with: aRule withArgs: args withPlaybackDebugging: debugging

	| input matcher ans|
	input := OM2LazyStream for: aReadStream withPos: 1.
	matcher := self new initInput: input.
	[
		matcher apply: #empty withArgs: args.
		ans := matcher apply: aRule.
		matcher apply: #end.
		^ ans
	] on: OM2Fail do: [:e |
		| curr prev prevPrev |
		debugging ifFalse: [e signal].
		curr := input.
		prev := nil.
		prevPrev := nil.
		[curr notNil] whileTrue: [
			prevPrev := prev.
			prev := curr.
			curr := curr basicTail
		].
		curr := prevPrev ifNotNil: [prevPrev] ifNil: [prev].
		self inform: 'will halt each time matcher reaches ', curr printString.
		matcher
			initInput: input;
			forgetEverything.
		curr ifNil: [self error: 'you''ve found a bug -- please tell Alex'].
		curr become: (OM2StreamDebugger for: curr copy).
		matcher haltingPoint: curr.
		matcher apply: #empty withArgs: args.
		ans := matcher apply: aRule.
		matcher apply: #end.
		^ ans
	]! !

!OMeta2Base class methodsFor: 'as yet unclassified' stamp: 'tak 4/7/2009 14:58'!
matcherOn: aReadStream

	| input matcher |
	input := OM2LazyStream for: aReadStream withPos: 1.
	matcher := self new initInput: input.
	^ matcher! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/16/2009 15:13'!
anything

	| ans |
	ans := input head.
	input := input tail.
	^ ans! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/16/2009 14:01'!
apply

	| aRule |
	aRule := self apply: #anything.
	^ self apply: aRule! !

!OMeta2Base methodsFor: 'rule application' stamp: 'warf 2/27/2009 14:09'!
apply: aRule

	" A memoRec is an association whose key is the answer,
	  and whose value is the next input. Failers pretend to
     be memoRecs, but throw a fail in response to #value "
	| memo memoRec |
	input == haltingPoint ifTrue: [self halt].
	memo := input memo.
	memoRec := memo at: aRule ifAbsent: [nil].
	memoRec ifNil: [
		| origInput failer ans |
		origInput := input.
		failer := OM2Failer new.
		memo at: aRule put: failer.
		ans := self perform: aRule.
		memoRec := ans -> input.
		memo at: aRule put: memoRec.
		failer used ifTrue: [
			" left recursion detected "
			| sentinel keepGoing |
			sentinel := input.
			keepGoing := true.
			[keepGoing] whileTrue: [
				[
					input := origInput.
					ans := self perform: aRule.
					input == sentinel ifTrue: [OMeta2Fail signal].
					memoRec key: ans value: input.
				] on: OM2Fail do: [keepGoing := false]
			]
		]		
	].
	input := memoRec value.
	^ memoRec key! !

!OMeta2Base methodsFor: 'rule application' stamp: 'warf 2/16/2009 13:39'!
apply: aRule withArgs: args

	args reverseDo: [:a | input := OM2Stream new initHead: a tail: input].
	^ self perform: aRule! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/19/2009 23:17'!
consumedBy: aBlock

	| origInput i ws |
	origInput := input.
	aBlock value.
	ws := WriteStream on: origInput inputSpecies new.
	i := origInput.
	[i == input] whileFalse: [
		ws nextPut: i head.
		i := i tail
	].
	^ ws contents! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 4/3/2010 00:37'!
empty

	^ true! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 4/3/2010 00:38'!
firstAndRest

	| first rest |
	first := self apply: #anything.
	rest := self apply: #anything.
	^ self genericMany: [self apply: rest] into: (OrderedCollection with: (self apply: first))! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/16/2009 14:11'!
foreign

	| aGrammar aRule g ans |
	aGrammar := self apply: #anything.
	aRule := self apply: #anything.
	g := aGrammar new initInput: (OM2StreamProxy for: input).
	ans := g apply: aRule.
	input := g input target.
	^ ans! !

!OMeta2Base methodsFor: 'forgetting' stamp: 'warf 2/26/2009 13:17'!
forgetEverything

	input transitiveForgetEverything.
	om2streams valuesDo: [:s | s transitiveForgetEverything]! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/26/2009 11:33'!
form: aBlock

	| v origInput |
	v := self apply: #anything.
	self pred: (v isCollection and: [v isSequenceable and: [v isSymbol not]]).
	origInput := input.
	input := om2streams at: v ifAbsentPut: [OM2LazyStream for: v readStream withPos: 1].
	aBlock value.
	self apply: #end.
	input := origInput.
	^ v! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/16/2009 15:19'!
genericMany: aBlock into: anOrderedCollection

	[
		| origInput |
		origInput := input.
		[anOrderedCollection addLast: aBlock value] on: OM2Fail do: [
			input := origInput.
			^ anOrderedCollection
		].
		true
	] whileTrue! !

!OMeta2Base methodsFor: 'initialize-release' stamp: 'warf 2/27/2009 11:25'!
haltingPoint: anOM2Stream

	haltingPoint := anOM2Stream! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/20/2009 14:57'!
indexConsumedBy: aBlock

	| from to |
	from := self pos.
	aBlock value.
	to := self pos.
	^ from -> to! !

!OMeta2Base methodsFor: 'initialize-release' stamp: 'warf 2/16/2009 14:06'!
initInput: i

	input := i! !

!OMeta2Base methodsFor: 'initialize-release' stamp: 'warf 2/26/2009 11:16'!
initialize

	super initialize.
	om2streams := IdentityDictionary new! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/16/2009 14:10'!
input

	^ input! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/16/2009 13:48'!
lookahead: aBlock

	| origInput ans |
	origInput := input.
	ans := aBlock value.
	input := origInput.
	^ ans! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/16/2009 13:56'!
many1: aBlock

	^ self genericMany: aBlock into: (OrderedCollection with: aBlock value)! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/16/2009 13:56'!
many: aBlock

	^ self genericMany: aBlock into: OrderedCollection new! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/25/2009 11:28'!
not: aBlock

	| origInput |
	origInput := input.
	[aBlock value] on: OM2Fail do: [
		input := origInput.
		^ true
	].
	OMeta2Fail signal! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/25/2009 11:28'!
ometaOr: choices

	| origInput |
	origInput := input.
	choices do: [:choice |
		input := origInput.
		[^ choice value] on: OM2Fail do: []
	].
	OMeta2Fail signal! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/20/2009 14:33'!
opt: aBlock

	^ self ometaOr: {
		[aBlock value].
		[nil]
	}! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/20/2009 14:48'!
pos

	^ input pos! !

!OMeta2Base methodsFor: 'private' stamp: 'warf 2/25/2009 11:28'!
pred: aBooleanValue

	" may want to have the compiler inline this automatically, for performance "
	aBooleanValue ifTrue: [^ true].
	OMeta2Fail signal! !

!OMeta2Base methodsFor: 'rules-basic' stamp: 'warf 2/16/2009 16:16'!
seq

	| xs |
	xs := self apply: #anything.
	xs do: [:x |
		" may want to inline #apply:withArgs: below as an
		  optimization, since this rule gets used a lot "
		self apply: #exactly withArgs: {x}
	].
	^ xs
	! !

!OMeta2Base methodsFor: 'rule application' stamp: 'warf 2/16/2009 15:03'!
super: superclass apply: aRule withArgs: args

	args reverseDo: [:a | input := OM2Stream new initHead: a tail: input].
	^ self perform: aRule withArguments: #() inSuperclass: superclass! !

MethodNode subclass: #OMeta2MethodNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OMeta2'!

!OMeta2MethodNode methodsFor: 'debugger support' stamp: 'hmm 8/25/2012 14:28'!
schematicTempNamesString
	"The decompiler wants a list of temp names. However, this mechanism depends on a number of assumptions that do not hold with OMeta2 generated methods. Therefore we simply skip the temp names, letting the decompiler build generic ones."
	^nil! !
OM2Fail initialize!
OM2PostloadPackageInfo initialize!
OM2PreloadPackageInfo initialize!
O2SqueakRecognizer initialize!
